# TODO: Need to make it singleton, one class per service
class Constants:

    def __init__(self):
        self.__parameter_map = {
            "SLACK_URL": "/SLACK/WEBHOOK_URL/ALERTS",
            "ENV": "STG",
            "REGION": "us-west-2"
        }

    def set_key_value(self, key, value):
        self.__parameter_map[key] = value

    def get_value_by_key(self, key):
        self.__parameter_map.get(key, None)


CONSTANTS_INSTANCE = Constants()



from .constants import *
import requests
import json
import boto3

import logging
log = logging.getLogger(__name__)


def notify_slack(subject, body, tags=None):
    """
    send a message to a slack based on an environment
    :param subject: meaningful title of the notification
    :param body: notification message
    :param tags: any extra information need to send to slack
    :return:
    """
    slack_url = CONSTANTS_INSTANCE.get_value_by_key("SLACK_URL")
    region = CONSTANTS_INSTANCE.get_value_by_key("REGION")
    env = CONSTANTS_INSTANCE.get_value_by_key("ENV")

    ssm = boto3.client('ssm', region_name=region)

    slack_url = ssm.get_parameter(Name=slack_url, WithDecryption=True).get('Parameter').get('Value')

    if tags is None:
        tags = [f"Running in {env}"]
    data = {"text": f"{env} \n {subject}\n {body}\n {tags}"}
    resp = requests.post(slack_url, data=json.dumps(data), headers={'Content-Type': 'application/json'})
    return resp


def get_secret_key_details(secret_name, tags=None):
    """
    will try to get details of the secret key by using the client
    :param secret_name: name of the secret key
    :param tags: Any tags to add while notifying slack?
    :return: json response
    """
    try:
        region = CONSTANTS_INSTANCE.get_value_by_key("REGION")
        env = CONSTANTS_INSTANCE.get_value_by_key("ENV")

        log.info(f'Making a secret call to manager with key {secret_name} in region {region} having environment {env}')

        session = boto3.session.Session()
        client = session.client(
            service_name='secretsmanager',
            region_name=region,
        )
        secret_value_response = client.get_secret_value(
            SecretId=secret_name
        )
        response = secret_value_response['SecretString']
        return response
    except Exception as ex:
        log.exception(f"Exception occurred while fetching details for key {secret_name} : {ex}")
        notify_slack(subject=f"Exception occurred while fetching details for key {secret_name}", body=f"{ex}",
                     tags=tags)
        raise



from setuptools import find_packages, setup

# Will create package of a version mentioned in the setup.
# Please append -stg when creating package for testing.

setup(
    name='gsg-utils-stg',
    packages=find_packages(include=['gsg_utils']),
    version='0.0.4',
    description='Library that will contain all utility functions that are accessible by other services.',
    url='https://bitbucket.org/global-savings-group/pouch-utils-library/src/master/',
    author='Pouch-devs',
    license='Pouch',
    install_requires=[
        'requests',
        'boto3'
    ],
    setup_requires=[
        'pytest-runner'
    ],
    tests_require=[
        'pytest'
    ],
    test_suite='tests',
)
